//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "download.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "IdAntiFreeze"
#pragma link "IdAntiFreezeBase"
#pragma link "IdBaseComponent"
#pragma link "IdComponent"
#pragma link "IdHTTP"
#pragma link "IdTCPClient"
#pragma link "IdTCPConnection"
#pragma resource "*.dfm"
TfrmDownload *frmDownload;
//---------------------------------------------------------------------------
__fastcall TfrmDownload::TfrmDownload(TComponent* Owner)
        : TForm(Owner)
{
}
//---------------------------------------------------------------------------
void __fastcall TfrmDownload::FormCreate(TObject *Sender)
{
   Left = (Screen->Width) / 2 - (Width / 2);
   Top = (Screen->Height) / 2 - (Height / 2);
   bytesExpected = 0;
}
//---------------------------------------------------------------------------
void __fastcall TfrmDownload::HTTPStatus(TObject *axSender,
      const TIdStatus axStatus, const AnsiString asStatusText)
{
   lblStatus->Caption = "Status: " + asStatusText;
}
//---------------------------------------------------------------------------
void __fastcall TfrmDownload::HTTPWork(TObject *Sender,
      TWorkMode AWorkMode, const int AWorkCount)
{
   if (AWorkMode == wmRead)
      lblBytes->Caption = "Bytes Received: " + IntToStr(AWorkCount) + " of " + IntToStr(bytesExpected);
}
//---------------------------------------------------------------------------
void __fastcall TfrmDownload::HTTPDisconnected(TObject *Sender)
{
   Timer1->Enabled = true;
}
//---------------------------------------------------------------------------
void __fastcall TfrmDownload::Timer1Timer(TObject *Sender)
{
   Close();
}
//---------------------------------------------------------------------------
void __fastcall TfrmDownload::FormActivate(TObject *Sender)
{
   TFileStream *theReadings;
   try {
      theReadings = new TFileStream(ExtractFilePath(Application->ExeName) + "readings.dat", fmCreate | fmShareExclusive);
   }
   catch (EFCreateError &error)
   {
      Application->MessageBoxA("Please make sure that the \"readings.dat\" file does not have the Read Only attribute set.","Error",MB_OK);
      if (theReadings) delete theReadings;
      return;
   }

   // Set up the HTTP component.
   HTTP->Host = "www.vivace.f9.co.uk";
   HTTP->Request->UserAgent = "Daily Reading from Taize Client v1.2.0";

   try {
      // Initiate the HTTP connection.
      HTTP->Connect(IdTimeoutDefault);

      // Try to download the new readings file.
      HTTP->Get("/misc_files/tdr.dat", theReadings);

      // Disconnect from the server.
      HTTP->Disconnect();
   }
   catch (EIdSocketError &error)
   {
      if (error.LastError == 11001) // Can't open a socket.
      {
         lblStatus->Caption =  "Status: Unable to connect to server.";
         HTTP->Disconnect();
      }
      if (theReadings) delete theReadings;
   }
   catch (EIdProtocolReplyError &error)
   {
      switch (error.ReplyErrorCode)
      {
         case 301 :
            // Moved Permanently.
            lblStatus->Caption =  "Status: 301 - Moved Permanently.";
            break;
         case 400 :
            // 400 - Bad Request.
            lblStatus->Caption =  "Status: 400 - Bad Request.";
            break;
         case 401 :
            // 401 - Unauthorized.
            lblStatus->Caption =  "Status: 401 - Unauthorized.";
            break;
         case 403 :
            // 403 - Forbidden.
            lblStatus->Caption =  "Status: 403 - Forbidden.";
            break;
         case 404 :
            // 404 - Not Found.
            lblStatus->Caption =  "Status: 404 - Not Found.";
            break;
         case 500 :
            // 500 - Internal Server Error.
            lblStatus->Caption =  "Status: 500 - Internal Server Error.";
            break;
         case 503 :
            // 503 - Service Unavailable.
            lblStatus->Caption =  "Status: 503 - Service Unavailable.";
            break;
         default :
            // Unknown Error.
            lblStatus->Caption =  "Status: An Unknown error occurred.";
      }
      HTTP->Disconnect();
      if (theReadings) delete theReadings;
   }
   catch (EExternalException &error)
   {
      lblStatus->Caption =  "Status: " + error.Message;
      HTTP->Disconnect();
      if (theReadings) delete theReadings;
   }
   if (theReadings) delete theReadings;
}
//---------------------------------------------------------------------------
void __fastcall TfrmDownload::HTTPWorkBegin(TObject *Sender,
      TWorkMode AWorkMode, const int AWorkCountMax)
{
   if (AWorkMode == wmRead)
   {
      bytesExpected = AWorkCountMax;
      lblBytes->Caption = "Bytes Received: 0 of " + IntToStr(bytesExpected);
   }   
}
//---------------------------------------------------------------------------
void __fastcall TfrmDownload::HTTPWorkEnd(TObject *Sender,
      TWorkMode AWorkMode)
{
   if (AWorkMode == wmRead)
      lblBytes->Caption = "Bytes Received: " + IntToStr(bytesExpected) + " of " +IntToStr(bytesExpected);

}
//---------------------------------------------------------------------------

